@@ -1,6 +1,6 @@
 // Copyright (c) 2017-present PyO3 Project and Contributors
 
-//! Conversions between various states of Rust and Python types and their wrappers.
+//! Defines conversions between Rust and Python types.
 use crate::err::{self, PyDowncastError, PyResult};
 use crate::type_object::PyTypeInfo;
 use crate::types::PyTuple;
@@ -17,7 +17,7 @@ use std::ptr::NonNull;
 /// # Examples
 ///
 /// ```
-/// use pyo3::{AsPyPointer, prelude::*};
+/// use pyo3::{prelude::*, AsPyPointer};
 /// Python::with_gil(|py| {
 ///     let dict = pyo3::types::PyDict::new(py);
 ///     // All native object wrappers implement AsPyPointer!!!
@@ -64,13 +64,7 @@ where
     T: AsPyPointer,
 {
     fn into_ptr(self) -> *mut ffi::PyObject {
-        let ptr = self.as_ptr();
-        if !ptr.is_null() {
-            unsafe {
-                ffi::Py_INCREF(ptr);
-            }
-        }
-        ptr
+        unsafe { ffi::_Py_XNewRef(self.as_ptr()) }
     }
 }
 
@@ -153,8 +147,8 @@ where
 ///
 /// #[pyclass]
 /// struct Number {
-///    #[pyo3(get, set)]
-///    value: i32,
+///     #[pyo3(get, set)]
+///     value: i32,
 /// }
 /// ```
 /// Python code will see this as an instance of the `Number` class with a `value` attribute.
@@ -167,14 +161,14 @@ where
 /// use pyo3::prelude::*;
 ///
 /// struct Number {
-///   value: i32,
+///     value: i32,
 /// }
 ///
 /// impl IntoPy<PyObject> for Number {
 ///     fn into_py(self, py: Python) -> PyObject {
 ///         // delegates to i32's IntoPy implementation.
 ///         self.value.into_py(py)
-///    }
+///     }
 /// }
 /// ```
 /// Python code will see this as an `int` object.
@@ -189,7 +183,7 @@ where
 /// enum Value {
 ///     Integer(i32),
 ///     String(String),
-///     None
+///     None,
 /// }
 ///
 /// impl IntoPy<PyObject> for Value {
@@ -197,10 +191,22 @@ where
 ///         match self {
 ///             Self::Integer(val) => val.into_py(py),
 ///             Self::String(val) => val.into_py(py),
-///             Self::None => py.None()
+///             Self::None => py.None(),
 ///         }
-///    }
+///     }
 /// }
+/// # fn main() {
+/// #     Python::with_gil(|py| {
+/// #         let v = Value::Integer(73).into_py(py);
+/// #         let v = v.extract::<i32>(py).unwrap();
+/// #
+/// #         let v = Value::String("foo".into()).into_py(py);
+/// #         let v = v.extract::<String>(py).unwrap();
+/// #
+/// #         let v = Value::None.into_py(py);
+/// #         let v = v.extract::<Option<Vec<i32>>>(py).unwrap();
+/// #     });
+/// # }
 /// ```
 /// Python code will see this as any of the `int`, `string` or `None` objects.
 #[cfg_attr(docsrs, doc(alias = "IntoPyCallbackOutput"))]
@@ -551,38 +557,38 @@ mod tests {
 
     #[test]
     fn test_try_from() {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let list: &PyAny = vec![3, 6, 5, 4, 7].to_object(py).into_ref(py);
-        let dict: &PyAny = vec![("reverse", true)].into_py_dict(py).as_ref();
+        Python::with_gil(|py| {
+            let list: &PyAny = vec![3, 6, 5, 4, 7].to_object(py).into_ref(py);
+            let dict: &PyAny = vec![("reverse", true)].into_py_dict(py).as_ref();
 
-        assert!(PyList::try_from(list).is_ok());
-        assert!(PyDict::try_from(dict).is_ok());
+            assert!(PyList::try_from(list).is_ok());
+            assert!(PyDict::try_from(dict).is_ok());
 
-        assert!(PyAny::try_from(list).is_ok());
-        assert!(PyAny::try_from(dict).is_ok());
+            assert!(PyAny::try_from(list).is_ok());
+            assert!(PyAny::try_from(dict).is_ok());
+        });
     }
 
     #[test]
     fn test_try_from_exact() {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let list: &PyAny = vec![3, 6, 5, 4, 7].to_object(py).into_ref(py);
-        let dict: &PyAny = vec![("reverse", true)].into_py_dict(py).as_ref();
+        Python::with_gil(|py| {
+            let list: &PyAny = vec![3, 6, 5, 4, 7].to_object(py).into_ref(py);
+            let dict: &PyAny = vec![("reverse", true)].into_py_dict(py).as_ref();
 
-        assert!(PyList::try_from_exact(list).is_ok());
-        assert!(PyDict::try_from_exact(dict).is_ok());
+            assert!(PyList::try_from_exact(list).is_ok());
+            assert!(PyDict::try_from_exact(dict).is_ok());
 
-        assert!(PyAny::try_from_exact(list).is_err());
-        assert!(PyAny::try_from_exact(dict).is_err());
+            assert!(PyAny::try_from_exact(list).is_err());
+            assert!(PyAny::try_from_exact(dict).is_err());
+        });
     }
 
     #[test]
     fn test_try_from_unchecked() {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let list = PyList::new(py, &[1, 2, 3]);
-        let val = unsafe { <PyList as PyTryFrom>::try_from_unchecked(list.as_ref()) };
-        assert_eq!(list, val);
+        Python::with_gil(|py| {
+            let list = PyList::new(py, &[1, 2, 3]);
+            let val = unsafe { <PyList as PyTryFrom>::try_from_unchecked(list.as_ref()) };
+            assert_eq!(list, val);
+        });
     }
 }
