@@ -7,10 +7,7 @@ use crate::{
     exceptions::{self, PyBaseException},
     ffi,
 };
-use crate::{
-    AsPyPointer, FromPyPointer, IntoPy, Py, PyAny, PyNativeType, PyObject, Python,
-    ToBorrowedObject, ToPyObject,
-};
+use crate::{AsPyPointer, IntoPy, Py, PyAny, PyObject, Python, ToBorrowedObject, ToPyObject};
 use std::borrow::Cow;
 use std::cell::UnsafeCell;
 use std::ffi::CString;
@@ -115,10 +112,10 @@ impl PyErr {
     ///
     /// # Examples
     /// ```rust
-    /// use pyo3::{Python, PyErr, IntoPy, exceptions::PyTypeError, types::PyType};
+    /// use pyo3::{exceptions::PyTypeError, types::PyType, IntoPy, PyErr, Python};
     /// Python::with_gil(|py| {
     ///     // Case #1: Exception instance
-    ///     let err = PyErr::from_instance(PyTypeError::new_err("some type error",).instance(py));
+    ///     let err = PyErr::from_instance(PyTypeError::new_err("some type error").instance(py));
     ///     assert_eq!(err.to_string(), "TypeError: some type error");
     ///
     ///     // Case #2: Exception type
@@ -127,7 +124,10 @@ impl PyErr {
     ///
     ///     // Case #3: Invalid exception value
     ///     let err = PyErr::from_instance("foo".into_py(py).as_ref(py));
-    ///     assert_eq!(err.to_string(), "TypeError: exceptions must derive from BaseException");
+    ///     assert_eq!(
+    ///         err.to_string(),
+    ///         "TypeError: exceptions must derive from BaseException"
+    ///     );
     /// });
     /// ```
     pub fn from_instance(obj: &PyAny) -> PyErr {
@@ -135,15 +135,13 @@ impl PyErr {
 
         let state = if unsafe { ffi::PyExceptionInstance_Check(ptr) } != 0 {
             PyErrState::Normalized(PyErrStateNormalized {
-                ptype: unsafe {
-                    Py::from_borrowed_ptr(obj.py(), ffi::PyExceptionInstance_Class(ptr))
-                },
+                ptype: obj.get_type().into(),
                 pvalue: unsafe { Py::from_borrowed_ptr(obj.py(), obj.as_ptr()) },
                 ptraceback: None,
             })
         } else if unsafe { ffi::PyExceptionClass_Check(obj.as_ptr()) } != 0 {
             PyErrState::FfiTuple {
-                ptype: unsafe { Some(Py::from_borrowed_ptr(obj.py(), ptr)) },
+                ptype: obj.into(),
                 pvalue: None,
                 ptraceback: None,
             }
@@ -160,9 +158,10 @@ impl PyErr {
     ///
     /// # Examples
     /// ```rust
-    /// use pyo3::{Python, PyErr, exceptions::PyTypeError, types::PyType};
+    /// use pyo3::{exceptions::PyTypeError, types::PyType, PyErr, Python};
+    ///
     /// Python::with_gil(|py| {
-    ///     let err = PyTypeError::new_err(("some type error",));
+    ///     let err: PyErr = PyTypeError::new_err(("some type error",));
     ///     assert_eq!(err.ptype(py), PyType::new::<PyTypeError>(py));
     /// });
     /// ```
@@ -175,10 +174,12 @@ impl PyErr {
     /// The object will be normalized first if needed.
     ///
     /// # Examples
+    ///
     /// ```rust
-    /// use pyo3::{Python, PyErr, exceptions::PyTypeError, types::PyType};
+    /// use pyo3::{exceptions::PyTypeError, PyErr, Python};
+    ///
     /// Python::with_gil(|py| {
-    ///     let err = PyTypeError::new_err(("some type error",));
+    ///     let err: PyErr = PyTypeError::new_err(("some type error",));
     ///     assert!(err.is_instance::<PyTypeError>(py));
     ///     assert_eq!(err.pvalue(py).to_string(), "some type error");
     /// });
@@ -193,7 +194,8 @@ impl PyErr {
     ///
     /// # Examples
     /// ```rust
-    /// use pyo3::{Python, PyErr, exceptions::PyTypeError, types::PyType};
+    /// use pyo3::{exceptions::PyTypeError, Python};
+    ///
     /// Python::with_gil(|py| {
     ///     let err = PyTypeError::new_err(("some type error",));
     ///     assert_eq!(err.ptraceback(py), None);
@@ -212,37 +214,95 @@ impl PyErr {
         unsafe { !ffi::PyErr_Occurred().is_null() }
     }
 
-    /// Retrieves the current error from the Python interpreter's global state.
+    /// Takes the current error from the Python interpreter's global state and clears the global
+    /// state. If no error is set, returns `None`.
     ///
-    /// The error is cleared from the Python interpreter.
-    /// If no error is set, returns a `SystemError`.
+    /// If the error is a `PanicException` (which would have originated from a panic in a pyo3
+    /// callback) then this function will resume the panic.
     ///
-    /// If the error fetched is a `PanicException` (which would have originated from a panic in a
-    /// pyo3 callback) then this function will resume the panic.
-    pub fn fetch(py: Python) -> PyErr {
-        unsafe {
+    /// Use this function when it is not known if an error should be present. If the error is
+    /// expected to have been set, for example from [PyErr::occurred] or by an error return value
+    /// from a C FFI function, use [PyErr::fetch].
+    pub fn take(py: Python) -> Option<PyErr> {
+        let (ptype, pvalue, ptraceback) = unsafe {
             let mut ptype: *mut ffi::PyObject = std::ptr::null_mut();
             let mut pvalue: *mut ffi::PyObject = std::ptr::null_mut();
             let mut ptraceback: *mut ffi::PyObject = std::ptr::null_mut();
             ffi::PyErr_Fetch(&mut ptype, &mut pvalue, &mut ptraceback);
 
-            let err = PyErr::new_from_ffi_tuple(py, ptype, pvalue, ptraceback);
+            // Convert to Py immediately so that any references are freed by early return.
+            let ptype = Py::from_owned_ptr_or_opt(py, ptype);
+            let pvalue = Py::from_owned_ptr_or_opt(py, pvalue);
+            let ptraceback = Py::from_owned_ptr_or_opt(py, ptraceback);
 
-            if ptype == PanicException::type_object(py).as_ptr() {
-                let msg: String = PyAny::from_borrowed_ptr_or_opt(py, pvalue)
-                    .and_then(|obj| obj.extract().ok())
-                    .unwrap_or_else(|| String::from("Unwrapped panic from Python code"));
+            // A valid exception state should always have a non-null ptype, but the other two may be
+            // null.
+            let ptype = match ptype {
+                Some(ptype) => ptype,
+                None => {
+                    debug_assert!(
+                        pvalue.is_none(),
+                        "Exception type was null but value was not null"
+                    );
+                    debug_assert!(
+                        ptraceback.is_none(),
+                        "Exception type was null but traceback was not null"
+                    );
+                    return None;
+                }
+            };
 
-                eprintln!(
-                    "--- PyO3 is resuming a panic after fetching a PanicException from Python. ---"
-                );
-                eprintln!("Python stack trace below:");
-                err.print(py);
+            (ptype, pvalue, ptraceback)
+        };
 
-                std::panic::resume_unwind(Box::new(msg))
+        if ptype.as_ptr() == PanicException::type_object(py).as_ptr() {
+            let msg: String = pvalue
+                .as_ref()
+                .and_then(|obj| obj.extract(py).ok())
+                .unwrap_or_else(|| String::from("Unwrapped panic from Python code"));
+
+            eprintln!(
+                "--- PyO3 is resuming a panic after fetching a PanicException from Python. ---"
+            );
+            eprintln!("Python stack trace below:");
+
+            unsafe {
+                use crate::conversion::IntoPyPointer;
+                ffi::PyErr_Restore(ptype.into_ptr(), pvalue.into_ptr(), ptraceback.into_ptr());
+                ffi::PyErr_PrintEx(0);
             }
 
-            err
+            std::panic::resume_unwind(Box::new(msg))
+        }
+
+        Some(PyErr::from_state(PyErrState::FfiTuple {
+            ptype,
+            pvalue,
+            ptraceback,
+        }))
+    }
+
+    /// Equivalent to [PyErr::take], but when no error is set:
+    ///  - Panics in debug mode.
+    ///  - Returns a `SystemError` in release mode.
+    ///
+    /// This behavior is consistent with Python's internal handling of what happens when a C return
+    /// value indicates an error occurred but the global error state is empty. (A lack of exception
+    /// should be treated as a bug in the code which returned an error code but did not set an
+    /// exception.)
+    ///
+    /// Use this function when the error is expected to have been set, for example from
+    /// [PyErr::occurred] or by an error return value from a C FFI function.
+    #[cfg_attr(all(debug_assertions, track_caller), track_caller)]
+    #[inline]
+    pub fn fetch(py: Python) -> PyErr {
+        const FAILED_TO_FETCH: &str = "attempted to fetch exception but none was set";
+        match PyErr::take(py) {
+            Some(err) => err,
+            #[cfg(debug_assertions)]
+            None => panic!("{}", FAILED_TO_FETCH),
+            #[cfg(not(debug_assertions))]
+            None => exceptions::PySystemError::new_err(FAILED_TO_FETCH),
         }
     }
 
@@ -279,27 +339,6 @@ impl PyErr {
         }
     }
 
-    /// Create a PyErr from an ffi tuple
-    ///
-    /// # Safety
-    /// - `ptype` must be a pointer to valid Python exception type object.
-    /// - `pvalue` must be a pointer to a valid Python object, or NULL.
-    /// - `ptraceback` must be a pointer to a valid Python traceback object, or NULL.
-    unsafe fn new_from_ffi_tuple(
-        py: Python,
-        ptype: *mut ffi::PyObject,
-        pvalue: *mut ffi::PyObject,
-        ptraceback: *mut ffi::PyObject,
-    ) -> PyErr {
-        // Note: must not panic to ensure all owned pointers get acquired correctly,
-        // and because we mustn't panic in normalize().
-        PyErr::from_state(PyErrState::FfiTuple {
-            ptype: Py::from_owned_ptr_or_opt(py, ptype),
-            pvalue: Py::from_owned_ptr_or_opt(py, pvalue),
-            ptraceback: Py::from_owned_ptr_or_opt(py, ptraceback),
-        })
-    }
-
     /// Prints a standard traceback to `sys.stderr`.
     pub fn print(&self, py: Python) {
         self.clone_ref(py).restore(py);
@@ -380,9 +419,9 @@ impl PyErr {
     ///
     /// # Examples
     /// ```rust
-    /// use pyo3::{Python, PyErr, exceptions::PyTypeError, types::PyType};
+    /// use pyo3::{exceptions::PyTypeError, PyErr, Python};
     /// Python::with_gil(|py| {
-    ///     let err = PyTypeError::new_err(("some type error",));
+    ///     let err: PyErr = PyTypeError::new_err(("some type error",));
     ///     let err_clone = err.clone_ref(py);
     ///     assert_eq!(err.ptype(py), err_clone.ptype(py));
     ///     assert_eq!(err.pvalue(py), err_clone.pvalue(py));
@@ -456,13 +495,8 @@ impl PyErr {
             ffi::PyErr_NormalizeException(&mut ptype, &mut pvalue, &mut ptraceback);
             let self_state = &mut *self.state.get();
             *self_state = Some(PyErrState::Normalized(PyErrStateNormalized {
-                ptype: Py::from_owned_ptr_or_opt(py, ptype)
-                    .unwrap_or_else(|| exceptions::PySystemError::type_object(py).into()),
-                pvalue: Py::from_owned_ptr_or_opt(py, pvalue).unwrap_or_else(|| {
-                    exceptions::PySystemError::new_err("Exception value missing")
-                        .instance(py)
-                        .into_py(py)
-                }),
+                ptype: Py::from_owned_ptr_or_opt(py, ptype).expect("Exception type missing"),
+                pvalue: Py::from_owned_ptr_or_opt(py, pvalue).expect("Exception value missing"),
                 ptraceback: PyObject::from_owned_ptr_or_opt(py, ptraceback),
             }));
 
@@ -569,6 +603,11 @@ mod tests {
     use crate::exceptions;
     use crate::{PyErr, Python};
 
+    #[test]
+    fn no_error() {
+        assert!(Python::with_gil(PyErr::take).is_none());
+    }
+
     #[test]
     fn set_valueerror() {
         Python::with_gil(|py| {
@@ -597,19 +636,29 @@ mod tests {
         })
     }
 
+    #[test]
+    fn set_typeerror() {
+        let gil = Python::acquire_gil();
+        let py = gil.python();
+        let err: PyErr = exceptions::PyTypeError::new_err(());
+        err.restore(py);
+        assert!(PyErr::occurred(py));
+        drop(PyErr::fetch(py));
+    }
+
     #[test]
     #[should_panic(expected = "new panic")]
     fn fetching_panic_exception_resumes_unwind() {
         use crate::panic::PanicException;
 
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let err: PyErr = PanicException::new_err("new panic");
-        err.restore(py);
-        assert!(PyErr::occurred(py));
+        Python::with_gil(|py| {
+            let err: PyErr = PanicException::new_err("new panic");
+            err.restore(py);
+            assert!(PyErr::occurred(py));
 
-        // should resume unwind
-        let _ = PyErr::fetch(py);
+            // should resume unwind
+            let _ = PyErr::fetch(py);
+        });
     }
 
     #[test]
@@ -621,42 +670,42 @@ mod tests {
         //     traceback: Some(<traceback object at 0x..)"
         // }
 
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let err = py
-            .run("raise Exception('banana')", None, None)
-            .expect_err("raising should have given us an error");
+        Python::with_gil(|py| {
+            let err = py
+                .run("raise Exception('banana')", None, None)
+                .expect_err("raising should have given us an error");
 
-        let debug_str = format!("{:?}", err);
-        assert!(debug_str.starts_with("PyErr { "));
-        assert!(debug_str.ends_with(" }"));
+            let debug_str = format!("{:?}", err);
+            assert!(debug_str.starts_with("PyErr { "));
+            assert!(debug_str.ends_with(" }"));
 
-        // strip "PyErr { " and " }"
-        let mut fields = debug_str["PyErr { ".len()..debug_str.len() - 2].split(", ");
+            // strip "PyErr { " and " }"
+            let mut fields = debug_str["PyErr { ".len()..debug_str.len() - 2].split(", ");
 
-        assert_eq!(fields.next().unwrap(), "type: <class 'Exception'>");
-        if py.version_info() >= (3, 7) {
-            assert_eq!(fields.next().unwrap(), "value: Exception('banana')");
-        } else {
-            // Python 3.6 and below formats the repr differently
-            assert_eq!(fields.next().unwrap(), ("value: Exception('banana',)"));
-        }
+            assert_eq!(fields.next().unwrap(), "type: <class 'Exception'>");
+            if py.version_info() >= (3, 7) {
+                assert_eq!(fields.next().unwrap(), "value: Exception('banana')");
+            } else {
+                // Python 3.6 and below formats the repr differently
+                assert_eq!(fields.next().unwrap(), ("value: Exception('banana',)"));
+            }
 
-        let traceback = fields.next().unwrap();
-        assert!(traceback.starts_with("traceback: Some(<traceback object at 0x"));
-        assert!(traceback.ends_with(">)"));
+            let traceback = fields.next().unwrap();
+            assert!(traceback.starts_with("traceback: Some(<traceback object at 0x"));
+            assert!(traceback.ends_with(">)"));
 
-        assert!(fields.next().is_none());
+            assert!(fields.next().is_none());
+        });
     }
 
     #[test]
     fn err_display() {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let err = py
-            .run("raise Exception('banana')", None, None)
-            .expect_err("raising should have given us an error");
-        assert_eq!(err.to_string(), "Exception: banana");
+        Python::with_gil(|py| {
+            let err = py
+                .run("raise Exception('banana')", None, None)
+                .expect_err("raising should have given us an error");
+            assert_eq!(err.to_string(), "Exception: banana");
+        });
     }
 
     #[test]
