@@ -1,6 +1,6 @@
 // Copyright (c) 2017-present PyO3 Project and Contributors
 use crate::conversion::{PyTryFrom, ToBorrowedObject};
-use crate::err::{PyDowncastError, PyErr, PyResult};
+use crate::err::{self, PyDowncastError, PyErr, PyResult};
 use crate::gil;
 use crate::pycell::{PyBorrowError, PyBorrowMutError, PyCell};
 use crate::types::{PyDict, PyTuple};
@@ -19,6 +19,7 @@ use std::ptr::NonNull;
 /// types.
 pub unsafe trait PyNativeType: Sized {
     /// Returns a GIL marker constrained to the lifetime of this type.
+    #[inline]
     fn py(&self) -> Python {
         unsafe { Python::assume_gil_acquired() }
     }
@@ -83,44 +84,74 @@ pub unsafe trait PyNativeType: Sized {
 /// [`Py`]`<T>` can be used to get around this by converting `dict` into a GIL-independent reference:
 ///
 /// ```rust
-/// # use pyo3::prelude::*;
-/// # use pyo3::types::PyDict;
-/// #
+/// use pyo3::prelude::*;
+/// use pyo3::types::PyDict;
+///
 /// #[pyclass]
 /// struct Foo {
 ///     inner: Py<PyDict>,
 /// }
 ///
+/// #[pymethods]
 /// impl Foo {
-///     fn new() -> Foo {
+///     #[new]
+///     fn __new__() -> Foo {
 ///         Python::with_gil(|py| {
 ///             let dict: Py<PyDict> = PyDict::new(py).into();
 ///             Foo { inner: dict }
 ///         })
 ///     }
 /// }
+/// #
+/// # fn main() -> PyResult<()> {
+/// #     Python::with_gil(|py| {
+/// #         let m = pyo3::types::PyModule::new(py, "test")?;
+/// #         m.add_class::<Foo>()?;
+/// #
+/// #         let foo: &PyCell<Foo> = pyo3::PyTryFrom::try_from(m.getattr("Foo")?.call0()?)?;
+/// #         let dict = &foo.borrow().inner;
+/// #         let dict: &PyDict = dict.as_ref(py);
+/// #
+/// #         Ok(())
+/// #     })
+/// # }
 /// ```
 ///
 /// This can also be done with other pyclasses:
 /// ```rust
-/// # use pyo3::prelude::*;
-/// #
+/// use pyo3::prelude::*;
+///
 /// #[pyclass]
-/// struct Bar {/* fields omitted */}
+/// struct Bar {/* ... */}
 ///
 /// #[pyclass]
 /// struct Foo {
 ///     inner: Py<Bar>,
 /// }
 ///
+/// #[pymethods]
 /// impl Foo {
-///     fn new() -> PyResult<Foo> {
+///     #[new]
+///     fn __new__() -> PyResult<Foo> {
 ///         Python::with_gil(|py| {
 ///             let bar: Py<Bar> = Py::new(py, Bar {})?;
 ///             Ok(Foo { inner: bar })
 ///         })
 ///     }
 /// }
+/// #
+/// # fn main() -> PyResult<()> {
+/// #     Python::with_gil(|py| {
+/// #         let m = pyo3::types::PyModule::new(py, "test")?;
+/// #         m.add_class::<Foo>()?;
+/// #
+/// #         let foo: &PyCell<Foo> = pyo3::PyTryFrom::try_from(m.getattr("Foo")?.call0()?)?;
+/// #         let bar = &foo.borrow().inner;
+/// #         let bar: &Bar = &*bar.borrow(py);
+/// #
+/// #         Ok(())
+/// #     })
+/// # }
 /// ```
 ///
 /// # Example: Shared ownership of Python objects
@@ -133,9 +164,9 @@ pub unsafe trait PyNativeType: Sized {
 /// [`Py::clone_ref`] will be faster if you happen to be already holding the GIL.
 ///
 /// ```rust
+/// use pyo3::conversion::AsPyPointer;
 /// use pyo3::prelude::*;
 /// use pyo3::types::PyDict;
-/// use pyo3::conversion::AsPyPointer;
 ///
 /// # fn main() {
 /// Python::with_gil(|py| {
@@ -165,6 +196,19 @@ pub unsafe trait PyNativeType: Sized {
 /// implement the [`PyGCProtocol`](crate::class::gc::PyGCProtocol). If your pyclass
 /// contains other Python objects you should implement this protocol to avoid leaking memory.
 ///
+/// # A note on Python reference counts
+///
+/// Dropping a [`Py`]`<T>` will eventually decrease Python's reference count
+/// of the pointed-to variable, allowing Python's garbage collector to free
+/// the associated memory, but this may not happen immediately.  This is
+/// because a [`Py`]`<T>` can be dropped at any time, but the Python reference
+/// count can only be modified when the GIL is held.
+///
+/// If a [`Py`]`<T>` is dropped while its thread happens to be holding the
+/// GIL then the Python reference count will be decreased immediately.
+/// Otherwise, the reference count will be decreased the next time the GIL is
+/// reacquired.
+///
 /// # A note on `Send` and `Sync`
 ///
 /// Accessing this object is threadsafe, since any access to its API requires a [`Python<'py>`](crate::Python) token.
@@ -245,10 +289,10 @@ where
     /// # use pyo3::prelude::*;
     /// #
     /// #[pyclass]
-    /// struct MyClass { }
+    /// struct MyClass {}
     ///
     /// Python::with_gil(|py| {
-    ///     let my_class: Py<MyClass> = Py::new(py, MyClass { }).unwrap();
+    ///     let my_class: Py<MyClass> = Py::new(py, MyClass {}).unwrap();
     ///     let my_class_cell: &PyCell<MyClass> = my_class.as_ref(py);
     ///     assert!(my_class_cell.try_borrow().is_ok());
     /// });
@@ -290,7 +334,7 @@ where
     ///
     /// ```rust
     /// # use pyo3::prelude::*;
-    /// #
+    /// # #[allow(dead_code)] // This is just to show it compiles.
     /// fn new_py_any<'py>(py: Python<'py>, value: impl IntoPy<Py<PyAny>>) -> &'py PyAny {
     ///     let obj: Py<PyAny> = value.into_py(py);
     ///
@@ -427,9 +471,9 @@ impl<T> Py<T> {
     /// # Examples
     ///
     /// ```rust
+    /// use pyo3::conversion::AsPyPointer;
     /// use pyo3::prelude::*;
     /// use pyo3::types::PyDict;
-    /// use pyo3::conversion::AsPyPointer;
     ///
     /// # fn main() {
     /// Python::with_gil(|py| {
@@ -458,11 +502,8 @@ impl<T> Py<T> {
     /// This is equivalent to the Python expression `bool(self)`.
     pub fn is_true(&self, py: Python) -> PyResult<bool> {
         let v = unsafe { ffi::PyObject_IsTrue(self.as_ptr()) };
-        if v == -1 {
-            Err(PyErr::fetch(py))
-        } else {
-            Ok(v != 0)
-        }
+        err::error_on_minusone(py, v)?;
+        Ok(v != 0)
     }
 
     /// Extracts some type from the Python object.
@@ -863,30 +904,29 @@ impl PyObject {
 mod tests {
     use super::{Py, PyObject};
     use crate::types::PyDict;
-    use crate::{ffi, AsPyPointer, Python};
+    use crate::Python;
 
     #[test]
     fn test_call_for_non_existing_method() {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let obj: PyObject = PyDict::new(py).into();
-        assert!(obj.call_method0(py, "asdf").is_err());
-        assert!(obj
-            .call_method(py, "nonexistent_method", (1,), None)
-            .is_err());
-        assert!(obj.call_method0(py, "nonexistent_method").is_err());
-        assert!(obj.call_method1(py, "nonexistent_method", (1,)).is_err());
+        Python::with_gil(|py| {
+            let obj: PyObject = PyDict::new(py).into();
+            assert!(obj.call_method0(py, "asdf").is_err());
+            assert!(obj
+                .call_method(py, "nonexistent_method", (1,), None)
+                .is_err());
+            assert!(obj.call_method0(py, "nonexistent_method").is_err());
+            assert!(obj.call_method1(py, "nonexistent_method", (1,)).is_err());
+        });
     }
 
     #[test]
     fn py_from_dict() {
-        let dict: Py<PyDict> = {
-            let gil = Python::acquire_gil();
-            let py = gil.python();
+        let dict: Py<PyDict> = Python::with_gil(|py| {
             let native = PyDict::new(py);
             Py::from(native)
-        };
-        assert_eq!(unsafe { ffi::Py_REFCNT(dict.as_ptr()) }, 1);
+        });
+
+        assert_eq!(Python::with_gil(|py| dict.get_refcnt(py)), 1);
     }
 
     #[test]
